#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
import Commands
from math import *
from qubtree import Node

def SharedDatasets( *results ):
    # return [ [ds, ds, ..per result], ... ] match filename
    byname = {}

    dss = [ res.node.find('DataSet') for res in results ]
    for firstDS in dss:
        ds = firstDS
        while not ds.isNull:
            if ds.name in byname.keys():
                byname[ ds.name ].append(ds)
            else:
                byname[ ds.name ] = [ds]
            ds = ds.next('DataSet')
    
    shared = []

    if len(results) > 0:
        ds = results[0].node.find('DataSet')
        while not ds.isNull:
            dslist = byname[ ds.name ]
            if len(dslist) == len(results):
                shared.append( dslist )
            ds = ds.next('DataSet')
            
    return shared   

def SharedSegs( *datasets ):
    # datasets should all be for same file
    # return [ [seg, seg, ..per ds], ... ] match seg.data
    bysel = {}
    for ds in datasets:
        seg = ds.find('Segment')
        while not seg.isNull:
            sel = seg.data[:]
            if sel in bysel.keys():
                bysel[ sel ].append( seg )
            else:
                bysel[ sel ] = [seg]
            seg = seg.next('Segment')

    shared = []

    if len(datasets) > 0:
        seg = datasets[0].find('Segment')
        while not seg.isNull:
            seglist = bysel[ seg.data[:] ]
            if len(seglist) == len(datasets):
                shared.append( seglist )
            seg = seg.next('Segment')

    return shared        
    

def CompareLL():
    selRes   = QUB.ResultsView.selectedResults
    nres     = len(selRes)
    datasets = apply( SharedDatasets, selRes )
    llNames  = map( lambda res: "%s LL" % res.fileName, selRes )

    for res in selRes:
        if not 'LL' in res.keys():
            QUB.Report('CompareLL: one or more results have no LL')
            return None

    out    = Node( str(selRes[0].node['ResultType'].data) + ' CompareLL')
    out['ResultType'].data = out.name
    
    for dss in datasets:
        segs = apply( SharedSegs, dss )
        nseg = len(segs)
        if nseg > 0:
            ds = Node('DataSet')
            ds['FileName'].data = dss[0]['FileName'].data
            out.append(ds)

            for segset in segs:
                seg = Node('Segment')
                seg.data = segset[0].data
                ds.append(seg)

                for i in range(0, nres):
                    LL = segset[i]['LL'].data[0]
                    seg[ llNames[i] ].data = LL
                    if (i == 0) or (max < LL):
                        max = LL
                        max_i = i

                if nres == 2:
                    seg['LL Ratio'].data = segset[0]['LL'].data[0] - segset[1]['LL'].data[0]

                seg['SelectedColor'].data = 1 + max_i                

    if not out.find('DataSet').isNull:
        QUB.ResultsView.addResults(out)
    else:
        QUB.Report('CompareLL: no segments in common')

Commands.Register('MultiResults', 'Compare LL', CompareLL)
    
